import logging
from functools import partial

import numpy as np
import treefiles as tf

from gcma.io.test_functions import *
from gcma.wrapper import CMA


class CMATestFunctions(CMA):
    test_functions = ("sphere", "rastrigin", "bukin")

    def set_test(self, name, **kwargs):
        if hasattr(self, name):
            setattr(self, "energy", partial(getattr(self, name), **kwargs))
        else:
            log.error(f"Test function {name!r} not found")

    def energy(self, *args, **kwargs):
        log.error("energy: you must call `set_test` to attach a test function")
        raise NotImplementedError

    @d1
    def sphere(self, x):
        return sphere(x)

    @d1
    def rastrigin(self, x, a=10):
        return rastrigin(x, a)

    @d1
    def bukin(self, x):
        return bukin(x)

    @d1
    def rosenbrock(self, x):
        return rosenbrock(x)


search_space = {
    "sphere": [(-10, 10)],  # (0, 0)
    "rastrigin": [(-5.12, 5.12)],  # (0, 0)
    "bukin": [(-15, -5), (-3, 3)],  # (-10, 1)
}


log = logging.getLogger(__name__)

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    log = tf.get_logger()

    X = np.array([[-10, 1.00002]])
    aa = bukin(X)
    print(aa)
